<?php

namespace Drupal\node_alias_link_display\Plugin\Filter;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\filter\Attribute\Filter;
use Drupal\filter\FilterProcessResult;
use Drupal\filter\Plugin\FilterBase;
use Drupal\filter\Plugin\FilterInterface;
use Drupal\node\Entity\Node;
use Drupal\path_alias\AliasManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Filter that replaces canonical node URLs with alias URLs in anchor hrefs.
 *
 * This filter scans text for anchor tags linking to canonical node paths,
 * i.e., URLs of the form "/node/{nid}". If an alias exists for the node path,
 * it replaces the canonical URL in the anchor href with the alias URL.
 *
 * This transformation is irreversible.
 *
 * @property \Drupal\path_alias\AliasManagerInterface $entity
 */
#[Filter(
  id: "filter_node_alias_link_display",
  title: new TranslatableMarkup("Replace canonical node URLs with aliases"),
  description: new TranslatableMarkup('Replaces anchor tag href attributes linking to "/node/{nid}" with their corresponding URL aliases'),
  type: FilterInterface::TYPE_TRANSFORM_IRREVERSIBLE,
  weight: 100
)]
class NodeAliasLinkDisplayFilter extends FilterBase implements ContainerFactoryPluginInterface {

  /**
   * Constructs a NodeLinkAliasFilter object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\path_alias\AliasManagerInterface $aliasManager
   *   The alias manager service.
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   The language manager service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(
    array $configuration,
    string $plugin_id,
    $plugin_definition,
    protected AliasManagerInterface $aliasManager,
    protected LanguageManagerInterface $languageManager,
    protected EntityTypeManagerInterface $entityTypeManager,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('path_alias.manager'),
      $container->get('language_manager'),
      $container->get('entity_type.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function process($text, $langcode) {
    $text = preg_replace_callback(
      '#<a\s+[^>]*href=["\'](/node/(\d+))["\'][^>]*>.*?</a>#i',
      fn($matches) => $this->replaceNodeUrlCallback($matches, $langcode),
      $text
    );

    return new FilterProcessResult($text);
  }

  /**
   * Callback for preg_replace_callback to replace node URLs with aliases.
   *
   * @param array $matches
   *   The regex matches, where:
   *   - $matches[0] is the full anchor tag.
   *   - $matches[1] is the original URL (/node/{nid}).
   *   - $matches[2] is the node ID (nid).
   * @param string $langcode
   *   The language code for alias lookup.
   *
   * @return string
   *   The replaced anchor tag with alias URL or original if no alias found.
   */
  protected function replaceNodeUrlCallback(array $matches, string $langcode): string {
    $original = $matches[0];
    $url = $matches[1];
    $nid = $matches[2];

    if (is_numeric($nid)) {
      $node_storage = $this->entityTypeManager->getStorage('node');
      if (($node = $node_storage->load($nid)) instanceof Node) {
        $language = $this->languageManager->getLanguage($langcode);
        $node_url = $node->toUrl('canonical', ['language' => $language]);
        $alias = $node_url->toString();
        // To check if can be replaced with $url and test /azeaze in link.
        if ($alias !== "/node/$nid") {
          return str_replace($url, $alias, $original);
        }
      }
    }

    return $original;
  }

}
